@@ -21,6 +21,7 @@ import copy
 import datetime
 import errno
 import functools
+import locale
 import os
 import re
 import shutil
@@ -729,7 +730,8 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                         s = os.lstat(os.path.join(root, e))
                     # Covers certain unreadable symlink cases; see bgo#585895
                     except OSError as err:
-                        error_string = e + err.strerror
+                        error_string = e + err.strerror.decode(
+                            'utf8', 'replace')
                         self.model.add_error(it, error_string, pane)
                         continue
 
@@ -750,7 +752,8 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                             if err.errno == errno.ENOENT:
                                 error_string = e + ": Dangling symlink"
                             else:
-                                error_string = e + err.strerror
+                                error_string = e + err.strerror.decode(
+                                    'utf8', 'replace')
                             self.model.add_error(it, error_string, pane)
                             differences = True
                     elif stat.S_ISREG(s.st_mode):
@@ -956,12 +959,17 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                     misc.copytree(src, dst)
                     self.recursively_update(path)
             except (OSError, IOError, shutil.Error) as err:
+                if len(err.args) == 1 and isinstance(err.args[0], unicode):
+                    # str() raises on Exception instance with non-ascii unicode
+                    msg = err.args[0].encode('utf8', 'replace')
+                else:
+                    msg = str(err)
                 misc.error_dialog(
                     _("Error copying file"),
                     _("Couldn't copy %s\nto %s.\n\n%s") % (
-                        GLib.markup_escape_text(src),
-                        GLib.markup_escape_text(dst),
-                        GLib.markup_escape_text(str(err)),
+                        GLib.markup_escape_text(src).decode('utf8', 'replace'),
+                        GLib.markup_escape_text(dst).decode('utf8', 'replace'),
+                        GLib.markup_escape_text(msg).decode('utf8', 'replace'),
                     )
                 )
 
@@ -1319,6 +1327,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             all_present_same = self.file_compare(lof, regexes)
         different = 1
         one_isdir = [None for i in range(self.model.ntree)]
+        locale_encoding = locale.getpreferredencoding()
         for j in range(self.model.ntree):
             if mod_times[j]:
                 isdir = os.path.isdir( files[j] )
@@ -1346,6 +1355,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 TIME = self.model.column_index(COL_TIME, j)
                 mod_datetime = datetime.datetime.fromtimestamp(mod_times[j])
                 time_str = mod_datetime.strftime("%a %d %b %Y %H:%M:%S")
+                time_str = time_str.decode(locale_encoding, errors='replace')
                 self.model.set_value(it, TIME, time_str)
 
                 def natural_size(bytes):
